<?php

/**
 * CARDGATE Library for PHP
 *
 * This file is a base class for all payment methods of CARDGATE.
 * However, creating an instance of this class will redirect you to a payment screen of CARDGATE
 * where you will be able to choose from different payment methods.
 *
 * @author CARDGATE <support@cardgate.com>
 * @copyright Copyright (c) 2011, CARDGATE
 * @version 1.01
 */
// For Ubercart usage

class CARDGATE {

    var $version = "1.0.7";
    protected $siteID = 0;
    protected $hashKey = "";
    protected $orderID = "";
    protected $issuer = "";
    protected $country = "";
    protected $language = "";
    protected $currency = "";
    protected $amount = 0;
    protected $description = "";
    protected $paymentMethod = "";
    protected $reference = "";
    protected $logging = false;
    protected $loggingDirectory = ".";
    protected $pageData = NULL;

    /**
     * Constructor
     * @since Version 1.01
     * @param int $siteID This is the site ID that you can create in your CARDGATE backoffice account.
     * @param int $secretCode This is the key that belongs to your site ID.
     * @return CARDGATE
     */
    public function __construct( $siteID = NULL, $secretCode = NULL ) {
        $this->siteID = $siteID;
        $this->hashKey = $secretCode;

        if ( $this->siteID == NULL )
            throw new Exception( "Please provide a siteID" );
        if ( $this->hashKey == NULL )
            throw new Exception( "Please provide a secret code" );
    }

    /**
     * Returns the current API version
     * @access public
     * @since Version 1.01
     * @return string
     */
    public function GetAPIVersion() {
        return $this->version;
    }

    /**
     * Returns the API ID
     * @access public
     * @since Version 1.01
     * @return string
     */
    public function GetAPIID() {
        return $this->generateFingerPrint();
    }

    /**
     * Find for a string in an array of strings
     * @access protected
     * @param $collection An array of strings
     * @param $find The string that needs to be found in the collection
     * @return true|false TRUE if the string is found, otherwise FALSE
     */
    protected function inCollection( $collection, $find ) {
        foreach ( $collection as $item )
            if ( $find == $item )
                return true;
        return false;
    }

    /**
     * Set the reference
     * @access public
     * @return void
     */
    public function SetReference( $reference ) {
        $this->reference = intval( $reference );
        return;
    }

    /**
     * Enable/disable logging
     * @access public
     * @return void
     */
    public function SetLogging( $logging ) {
        $this->logging = $logging;
        return;
    }

    /**
     * Set logging directory
     * @access public
     * @return void
     */
    public function SetLoggingDirectory( $loggingDirectory ) {
        $this->loggingDirectory = $loggingDirectory;
        return;
    }

    /**
     * Sets the API URL
     * @access public
     * @return void
     */
    public function SetApiURL( $url ) {
        $this->apiURL = $url;
        return;
    }

    /**
     * Set stream method
     * @access public
     * @return void
     */
    public function SetStreamMethod( $streamMethod ) {
        $this->streamMethod = strtolower( $streamMethod );
        return;
    }

    /**
     * Appends text to a log file
     * @access protected
     * @return bool Returns TRUE if logging is enabled, otherwise FALSE
     */
    protected function doLogging( $line ) {
        if ( !$this->logging )
            return false;
        date_default_timezone_set( "Europe/Paris" );
        $filename = sprintf( "%s/#%s.log", $this->loggingDirectory, date( "Ymd", time() ) );
        $fp = @fopen( $filename, "a" );
        $line = sprintf( "%s - %s\r\n", date( "H:i", time() ), $line );
        @fwrite( $fp, $line );
        @fclose( $fp );

        return true;
    }

    public function VerifyPostback() {

        if ( $_SERVER['REQUEST_METHOD'] != 'POST' && !isset( $_POST['ref'] ) && !isset( $_POST['status'] ) )
            return false;

        $this->reference = $_POST['ref'];
        $this->status = $_POST['status'];
        $this->doLogging( sprintf( "Postback: %s", serialize( $_POST ) ) );
        $order = uc_order_load( $this->reference );

        $currency = $order->currency;
        $amount = $order->order_total * 100;

        $hashKey = variable_get( 'cardgate_hash_key', '' );
        $test = variable_get( 'cardgate_mode', '' );

        $hash = md5( ($test == 'test' ? 'TEST' : '') . $_POST['transactionid'] . $currency . $amount . $this->reference . $_POST['status'] . $hashKey );

        if ( $hash == $_POST['hash'] ) {
            return true;
        } else {
            return false;
        }
    }

    public function getNotifyUrl() {
        if ( !empty( $_SERVER['CGP_GATEWAY_URL'] ) ) {
            return $_SERVER['CGP_GATEWAY_URL'];
        } else {
            $test = variable_get( 'cardgate_mode', '' );
            if ( $test == 'live' ) {
                return "https://secure.curopayments.net/gateway/cardgate/";
            } else {
                return "https://secure-staging.curopayments.net/gateway/cardgate/";
            }
        }
    }

    public function getOrderData( $order ) {
        
        // See if an applicable tax exists
        foreach ( $order->line_items as $sKey => $aLineItem ) {
            if ( $aLineItem['type'] == 'tax' ) {
                $data = $aLineItem['data']['tax'];
                $tax = array();
                $tax['rate'] = $data->rate;
                $tax['shippable'] = $data->shippable;
                $tax['taxed_product_types'] = $data->taxed_product_types; // array
                $tax['taxed_line_items'] = $data->taxed_line_items;
            }
        }

        foreach ( $order->products as $sKey => $oItem ) {

            $price = $oItem->price * 100;
            $vat = 0;
            $vat_amount = $oItem->price * 100;

            if ( !empty( $tax ) ) {
                if ( in_array( $oItem->type, $tax['taxed_product_types'] ) ) {
                    $price = $price * (1 + $tax['rate']);
                    $vat = $tax['rate'] * 100;
                    $vat_amount = $oItem->price * $vat;
                }
            }

            $aCartItems[] = array(
                'quantity' => $oItem->qty,
                'sku' => $oItem->model,
                'name' => $oItem->title,
                'price' => round( $price, 0 ),
                'vat' => round( $vat, 0 ),
                'vat_amount' => round( $vat_amount, 0 ),
                'vat_inc' => true,
                'type' => 1 );
        }


        foreach ( $order->line_items as $sKey => $aLineItem ) {
            switch ( $aLineItem['type'] ) {
                case 'shipping':
                    $price = $aLineItem['amount'] * 100;
                    $vat = 0;
                    $vat_amount = $aLineItem['amount'] * 100;

                    if ( !empty( $tax ) ) {
                        if ( in_array( 'shipping', $tax['taxed_line_items'] ) ) {
                            $price = $price * (1 + $tax['rate']);
                            $vat = $tax['rate'] * 100;
                            $vat_amount = $aLineItem['amount'] * $vat;
                        }
                    }
                    $aCartItems[] = array(
                        'quantity' => 1,
                        'sku' => 'SHIPPING',
                        'name' => $aLineItem['title'],
                        'price' => round( $price, 0 ),
                        'vat' => round( $vat, 0 ),
                        'vat_amount' => round( $vat_amount, 0 ),
                        'vat_inc' => true,
                        'type' => 2 );
                    break;

                case 'coupon':
                    // $price = ($aLineItem['amount'] < 0 ) ? ( $aLineItem['amount'] * -1 ) * 100 : $aLineItem['amount'] * 100;
                    $price = $aLineItem['amount'] * 100;
                    $vat = 0;
                    $vat_amount = $aLineItem['amount'] * 100;

                    if ( !empty( $tax ) ) {
                        if ( in_array( 'coupon', $tax['taxed_line_items'] ) ) {
                            $price = $price * (1 + $tax['rate']);
                            $vat = $tax['rate'] * 100;
                            $vat_amount = $aLineItem['amount'] * $vat;
                        }
                    }
                    $aCartItems[] = array(
                        'quantity' => 1,
                        'sku' => 'DISCOUNT',
                        'name' => $aLineItem['title'],
                        'price' => round( $price, 0 ),
                        'vat' => round( $vat, 0 ),
                        'vat_amount' => round( $vat_amount, 0 ),
                        'vat_inc' => true,
                        'type' => 2 );
                    break;
            }
        }
        
        
        $data = array();
        $test = variable_get( 'cardgate_mode', '' );
        $my_country = array( 'country_id' => $order->billing_country );
        $b_country = uc_get_country_data( $my_country );
        $amount = uc_currency_format( $order->order_total, FALSE, FALSE, '.' ) * 100;
        $ref = $order->order_id;
        $hashKey = variable_get( 'cardgate_hash_key', '' );
        $hash = md5( ($test == 'live' ? '' : 'TEST') . $this->siteID . $amount . $ref . $hashKey );

        $data['test'] = ($test == 'test' ? 1 : 0);
        $data['option'] = substr( $order->payment_method, 4 );
        $data['suboption'] = ($data['option'] == 'ideal' ? $_SESSION['cardgate'] : '');
        $data['siteid'] = $this->siteID;
        $data['currency'] = variable_get( 'cardgate_currency', 'EUR' );
        $data['amount'] = $amount;
        $data['ref'] = $ref;
        $data['description'] = 'order_' . $order->order_id;
        $data['control_url'] = url( 'cart/cgp_response', array( 'absolute' => TRUE ) );
        $data['return_url'] = url( 'cart/cgp_success', array( 'absolute' => TRUE ) );
        $data['return_url_failed'] = url( 'cart/cgp_failure', array( 'absolute' => TRUE ) );
        $data['email'] = (isset( $order->primary_email ) ? $order->primary_email : '');
        $data['first_name'] = (isset( $order->billing_first_name ) ? $order->billing_first_name : '');
        $data['last_name'] = (isset( $order->billing_last_name ) ? $order->billing_last_name : '');
        $data['address'] = (isset( $order->billing_street1 ) ? $order->billing_street1 : '');
        $data['postal_code'] = (isset( $order->billing_postal_code ) ? $order->billing_postal_code : '');
        $data['city'] = (isset( $order->billing_city ) ? $order->billing_city : '');
        $data['country_code'] = $b_country[0]['country_iso_code_2'];
        $data['hash'] = $hash;
        $data['shop_name'] = 'DrupalUbercart';
        $data['plugin_name'] = 'Cardgate_Drupal';
        $data['plugin_version'] = $this->version;
        
        if ( count( $aCartItems ) > 0 ) {
            $data['cartitems'] = json_encode( $aCartItems, JSON_HEX_APOS | JSON_HEX_QUOT );
        }
        return $data;
    }

}
